/**
  ******************************************************************************
  * @file    usbd_cdc_vcp.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    22-July-2011
  * @brief   Generic media access Layer.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 

#ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED 
#pragma     data_alignment = 4 
#endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */

/* Includes ------------------------------------------------------------------*/
#include "usbd_cdc_vcp.h"
#include "MT9D111.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define MT9D111_RD_REG_CMD  0x01
#define MT9D111_WR_REG_CMD  0x02
#define MT9D111_RD_VAR_CMD  0x03
#define MT9D111_WR_VAR_CMD  0x04

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
uint8_t TxBuffer[64];
volatile uint16_t reg_value;
volatile uint16_t var_addr;
volatile uint16_t var_data;

/* These are external variables imported from CDC core to be used for IN 
   transfer management. */
extern uint8_t  APP_Rx_Buffer []; /* Write CDC received data in this buffer.
                                     These data will be sent over USB IN endpoint
                                     in the CDC core functions. */
extern uint32_t APP_Rx_ptr_in;    /* Increment this pointer or roll it back to
                                     start address when writing received data
                                     in the buffer APP_Rx_Buffer. */

/* Private function prototypes -----------------------------------------------*/

CDC_IF_Prop_TypeDef VCP_fops = 
{
  VCP_Init,
  VCP_DeInit,
  VCP_Ctrl,
  VCP_DataTx,
  VCP_DataRx
};

/* Private functions ---------------------------------------------------------*/
/**
  * @brief  VCP_Init
  *         Initializes the Media on the STM32
  * @param  None
  * @retval Result of the opeartion (USBD_OK in all cases)
  */
static uint16_t VCP_Init(void)
{ 
  return USBD_OK;
}

/**
  * @brief  VCP_DeInit
  *         DeInitializes the Media on the STM32
  * @param  None
  * @retval Result of the opeartion (USBD_OK in all cases)
  */
static uint16_t VCP_DeInit(void)
{
  return USBD_OK;
}


/**
  * @brief  VCP_Ctrl
  *         Manage the CDC class requests
  * @param  Cmd: Command code            
  * @param  Buf: Buffer containing command data (request parameters)
  * @param  Len: Number of data to be sent (in bytes)
  * @retval Result of the opeartion (USBD_OK in all cases)
  */
static uint16_t VCP_Ctrl (uint32_t Cmd, uint8_t* Buf, uint32_t Len)
{ 
  switch (Cmd)
  {
  case SEND_ENCAPSULATED_COMMAND:
    /* Not  needed for this driver */
    break;

  case GET_ENCAPSULATED_RESPONSE:
    /* Not  needed for this driver */
    break;

  case SET_COMM_FEATURE:
    /* Not  needed for this driver */
    break;

  case GET_COMM_FEATURE:
    /* Not  needed for this driver */
    break;

  case CLEAR_COMM_FEATURE:
    /* Not  needed for this driver */
    break;

  case SET_LINE_CODING:
    break;

  case GET_LINE_CODING:
    break;

  case SET_CONTROL_LINE_STATE:
    /* Not  needed for this driver */
    break;

  case SEND_BREAK:
    /* Not  needed for this driver */
    break;    
    
  default:
    break;
  }

  return USBD_OK;
}

/**
  * @brief  VCP_DataTx
  *         CDC received data to be send over USB IN endpoint are managed in 
  *         this function.
  * @param  Buf: Buffer of data to be sent
  * @param  Len: Number of data to be sent (in bytes)
  * @retval Result of the opeartion: USBD_OK if all operations are OK else VCP_FAIL
  */
uint16_t VCP_DataTx (uint8_t* Buffer, uint32_t Length)
{
	uint32_t i;

	//loop through buffer
  for(i = 0; i < Length; i++)
  {
    APP_Rx_Buffer[APP_Rx_ptr_in] = (uint8_t) Buffer[i];
		//increase pointer value
		APP_Rx_ptr_in++;
		/* To avoid buffer overflow */
		if(APP_Rx_ptr_in == APP_RX_DATA_SIZE)
		{
			APP_Rx_ptr_in = 0;
		}
  }
        
	return USBD_OK;
}

/**
  * @brief  VCP_DataRx
  *         Data received over USB OUT endpoint are sent over CDC interface 
  *         through this function.
  *           
  *         @note
  *         This function will block any OUT packet reception on USB endpoint 
  *         untill exiting this function. If you exit this function before transfer
  *         is complete on CDC interface (ie. using DMA controller) it will result 
  *         in receiving more data while previous ones are still not sent.
  *                 
  * @param  Buf: Buffer of data to be received
  * @param  Len: Number of data received (in bytes)
  * @retval Result of the opeartion: USBD_OK if all operations are OK else VCP_FAIL
  */
static uint16_t VCP_DataRx (uint8_t* Buffer, uint32_t Len)
{
	if(Buffer[0] == MT9D111_RD_REG_CMD){
		MT9D111_WriteReg(PAGE_SELECT_REG, Buffer[2]);   // Select Page
		reg_value = MT9D111_ReadReg(Buffer[1]);         // Select and Read Register

    TxBuffer[0] = reg_value >> 8;                   // MS Byte
    TxBuffer[1] = reg_value & 0xFF;                 // LS Byte
    VCP_DataTx(TxBuffer,2);                         // Write
    
  }
  else if(Buffer[0] == MT9D111_WR_REG_CMD){
		reg_value = (Buffer[3]<<8) + Buffer[4];         // Get 16-bit value

		MT9D111_WriteReg(PAGE_SELECT_REG, Buffer[2]);   // Select Page
		MT9D111_WriteReg(Buffer[1], reg_value);         // Select and Write Register */
	}
	else if(Buffer[0] == MT9D111_WR_VAR_CMD){
    var_addr = (Buffer[1]<<8) + Buffer[2];          // Get 16-bit address
		var_data = (Buffer[3]<<8) + Buffer[4];          // Get 16-bit value

		MT9D111_WriteReg(PAGE_SELECT_REG, 0x1);         // Select Page
    MT9D111_WriteReg(UC_VAR_ADDR_REG, var_addr);    // Select Variable
    MT9D111_WriteReg(UC_VAR_DATA_REG, var_data);    // Write Variable
  }
	else if(Buffer[0] == MT9D111_RD_VAR_CMD){
    var_addr = (Buffer[1]<<8) + Buffer[2];

		MT9D111_WriteReg(PAGE_SELECT_REG, 0x1);         // Select Page
    MT9D111_WriteReg(UC_VAR_ADDR_REG, var_addr);    // Select Variable
    var_data = MT9D111_ReadReg(UC_VAR_DATA_REG);    // Read Variable
    
    TxBuffer[0] = var_data >> 8;                    // MS Byte
    TxBuffer[1] = var_data & 0xFF;                  // LS Byte
    VCP_DataTx(TxBuffer,2);                         // Write
  }

	return USBD_OK;
}

/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
